@@ -23,6 +23,7 @@
 
 static cmdinfo_t project_cmd;
 static prid_t prid;
+static int recurse_depth = -1;
 
 enum {
 	CHECK_PROJECT	= 0x1,
@@ -31,11 +32,11 @@ enum {
 };
 
 #define EXCLUDED_FILE_TYPES(x) \
-	   S_ISCHR((x)) \
+	   (S_ISCHR((x)) \
 	|| S_ISBLK((x)) \
 	|| S_ISFIFO((x)) \
 	|| S_ISLNK((x)) \
-	|| S_ISSOCK((x))
+	|| S_ISSOCK((x)))
 
 static void
 project_help(void)
@@ -62,7 +63,7 @@ project_help(void)
 " Once this has been done, new files created in the tree will automatically\n"
 " be accounted to the tree based on their project identifier.  An attempt to\n"
 " create a hard link to a file in the tree will only succeed if the project\n"
-" identifier matches the project identifer for the tree.  The xfs_io utility\n"
+" identifier matches the project identifier for the tree.  The xfs_io utility\n"
 " can be used to set the project ID for an arbitrary file, but this can only\n"
 " be done by a privileged user.\n"
 "\n"
@@ -75,6 +76,15 @@ project_help(void)
 " which do not have the project ID of the rest of the tree, or if the inode\n"
 " flag is not set.\n"
 "\n"
+" The -p <path> option can be used to manually specify project path without\n"
+" need to create /etc/projects file. This option can be used multiple times\n"
+" to specify multiple paths. When using this option only one projid/name can\n"
+" be specified at command line. Note that /etc/projects is also used if exists.\n"
+"\n"
+" The -d <depth> option allows to descend at most <depth> levels of directories\n"
+" below the command line arguments. -d 0 means only apply the actions\n"
+" to the top level of the projects. -d -1 means no recursion limit (default).\n"
+"\n"
 " The /etc/projid and /etc/projects file formats are simple, and described\n"
 " on the xfs_quota man page.\n"
 "\n"));
@@ -90,7 +100,11 @@ check_project(
 	struct fsxattr		fsx;
 	int			fd;
 
+	if (recurse_depth >= 0 && data->level > recurse_depth)
+		return -1;
+
 	if (flag == FTW_NS ){
+		exitcode = 1;
 		fprintf(stderr, _("%s: cannot stat file %s\n"), progname, path);
 		return 0;
 	}
@@ -99,13 +113,15 @@ check_project(
 		return 0;
 	}
 
-	if ((fd = open(path, O_RDONLY|O_NOCTTY)) == -1)
+	if ((fd = open(path, O_RDONLY|O_NOCTTY)) == -1) {
+		exitcode = 1;
 		fprintf(stderr, _("%s: cannot open %s: %s\n"),
 			progname, path, strerror(errno));
-	else if ((xfsctl(path, fd, XFS_IOC_FSGETXATTR, &fsx)) < 0)
+	} else if ((xfsctl(path, fd, XFS_IOC_FSGETXATTR, &fsx)) < 0) {
+		exitcode = 1;
 		fprintf(stderr, _("%s: cannot get flags on %s: %s\n"),
 			progname, path, strerror(errno));
-	else {
+	} else {
 		if (fsx.fsx_projid != prid)
 			printf(_("%s - project identifier is not set"
 				 " (inode=%u, tree=%u)\n"),
@@ -129,7 +145,11 @@ clear_project(
 	struct fsxattr		fsx;
 	int			fd;
 
+	if (recurse_depth >= 0 && data->level > recurse_depth)
+		return -1;
+
 	if (flag == FTW_NS ){
+		exitcode = 1;
 		fprintf(stderr, _("%s: cannot stat file %s\n"), progname, path);
 		return 0;
 	}
@@ -139,10 +159,12 @@ clear_project(
 	}
 
 	if ((fd = open(path, O_RDONLY|O_NOCTTY)) == -1) {
+		exitcode = 1;
 		fprintf(stderr, _("%s: cannot open %s: %s\n"),
 			progname, path, strerror(errno));
 		return 0;
 	} else if (xfsctl(path, fd, XFS_IOC_FSGETXATTR, &fsx) < 0) {
+		exitcode = 1;
 		fprintf(stderr, _("%s: cannot get flags on %s: %s\n"),
 			progname, path, strerror(errno));
 		close(fd);
@@ -151,9 +173,11 @@ clear_project(
 
 	fsx.fsx_projid = 0;
 	fsx.fsx_xflags &= ~XFS_XFLAG_PROJINHERIT;
-	if (xfsctl(path, fd, XFS_IOC_FSSETXATTR, &fsx) < 0)
+	if (xfsctl(path, fd, XFS_IOC_FSSETXATTR, &fsx) < 0) {
+		exitcode = 1;
 		fprintf(stderr, _("%s: cannot clear project on %s: %s\n"),
 			progname, path, strerror(errno));
+	}
 	close(fd);
 	return 0;
 }
@@ -168,7 +192,11 @@ setup_project(
 	struct fsxattr		fsx;
 	int			fd;
 
+	if (recurse_depth >= 0 && data->level > recurse_depth)
+		return -1;
+
 	if (flag == FTW_NS ){
+		exitcode = 1;
 		fprintf(stderr, _("%s: cannot stat file %s\n"), progname, path);
 		return 0;
 	}
@@ -178,10 +206,12 @@ setup_project(
 	}
 
 	if ((fd = open(path, O_RDONLY|O_NOCTTY)) == -1) {
+		exitcode = 1;
 		fprintf(stderr, _("%s: cannot open %s: %s\n"),
 			progname, path, strerror(errno));
 		return 0;
 	} else if (xfsctl(path, fd, XFS_IOC_FSGETXATTR, &fsx) < 0) {
+		exitcode = 1;
 		fprintf(stderr, _("%s: cannot get flags on %s: %s\n"),
 			progname, path, strerror(errno));
 		close(fd);
@@ -190,9 +220,11 @@ setup_project(
 
 	fsx.fsx_projid = prid;
 	fsx.fsx_xflags |= XFS_XFLAG_PROJINHERIT;
-	if (xfsctl(path, fd, XFS_IOC_FSSETXATTR, &fsx) < 0)
+	if (xfsctl(path, fd, XFS_IOC_FSSETXATTR, &fsx) < 0) {
+		exitcode = 1;
 		fprintf(stderr, _("%s: cannot set project on %s: %s\n"),
 			progname, path, strerror(errno));
+	}
 	close(fd);
 	return 0;
 }
@@ -206,15 +238,15 @@ project_operations(
 	switch (type) {
 	case CHECK_PROJECT:
 		printf(_("Checking project %s (path %s)...\n"), project, dir);
-		nftw(dir, check_project, 100, FTW_PHYS|FTW_MOUNT|FTW_DEPTH);
+		nftw(dir, check_project, 100, FTW_PHYS|FTW_MOUNT);
 		break;
 	case SETUP_PROJECT:
 		printf(_("Setting up project %s (path %s)...\n"), project, dir);
-		nftw(dir, setup_project, 100, FTW_PHYS|FTW_MOUNT|FTW_DEPTH);
+		nftw(dir, setup_project, 100, FTW_PHYS|FTW_MOUNT);
 		break;
 	case CLEAR_PROJECT:
 		printf(_("Clearing project %s (path %s)...\n"), project, dir);
-		nftw(dir, clear_project, 100, FTW_PHYS|FTW_MOUNT|FTW_DEPTH);
+		nftw(dir, clear_project, 100, FTW_PHYS|FTW_MOUNT);
 		break;
 	}
 }
@@ -230,14 +262,16 @@ project(
 
 	fs_cursor_initialise(NULL, FS_PROJECT_PATH, &cursor);
 	while ((path = fs_cursor_next_entry(&cursor))) {
-		if (prid != path->fs_prid)
+		if (prid != path->fs_prid && path->fs_prid != -1)
 			continue;
 		project_operations(project, path->fs_dir, type);
 		count++;
 	}
 
-	printf(_("Processed %d %s paths for project %s\n"),
-		 count, projects_file, project);
+	printf(_("Processed %d (%s and cmdline) paths for project %s with "
+		"recursion depth %s (%d).\n"),
+		 count, projects_file, project,
+		 recurse_depth < 0 ? _("infinite") : _("limited"), recurse_depth);
 }
 
 static int
@@ -245,13 +279,22 @@ project_f(
 	int		argc,
 	char		**argv)
 {
-	int		c, type = 0;
+	int		c, type = 0, ispath = 0;
 
-	while ((c = getopt(argc, argv, "csC")) != EOF) {
+	while ((c = getopt(argc, argv, "cd:p:sC")) != EOF) {
 		switch (c) {
 		case 'c':
 			type = CHECK_PROJECT;
 			break;
+		case 'd':
+			recurse_depth = atoi(optarg);
+			if (recurse_depth < 0)
+				recurse_depth = -1;
+			break;
+		case 'p':
+			ispath = 1;
+			fs_table_insert_project_path(optarg, -1);
+			break;
 		case 's':
 			type = SETUP_PROJECT;
 			break;
@@ -271,18 +314,29 @@ project_f(
 		type = CHECK_PROJECT;
 
 	setprfiles();
-	if (access(projects_file, F_OK) != 0) {
+	if (!ispath && access(projects_file, F_OK) != 0) {
+		exitcode = 1;
 		fprintf(stderr, _("projects file \"%s\" doesn't exist\n"),
 			projects_file);
 		return 0;
 	}
 
+	if (ispath && argc - optind > 1) {
+		exitcode = 1;
+		fprintf(stderr, _("%s: only one projid/name can be specified "
+			    "when using -p <path>, %d found.\n"),
+			progname, argc - optind);
+		return 0;
+	}
+
         while (argc > optind) {
 		prid = prid_from_string(argv[optind]);
-		if (prid == -1)
-			fprintf(stderr, _("%s - no such project in %s\n"),
+		if (prid == -1) {
+			exitcode = 1;
+			fprintf(stderr, _("%s - no such project in %s "
+				    "or invalid project number\n"),
 				argv[optind], projects_file);
-		else
+		} else
 	                project(argv[optind], type);
 		optind++;
 	}
@@ -296,7 +350,7 @@ project_init(void)
 	project_cmd.name = _("project");
 	project_cmd.altname = _("tree");
 	project_cmd.cfunc = project_f;
-	project_cmd.args = _("[-c|-s|-C] project ...");
+	project_cmd.args = _("[-c|-s|-C|-d <depth>|-p <path>] project ...");
 	project_cmd.argmin = 1;
 	project_cmd.argmax = -1;
 	project_cmd.oneline = _("check, setup or clear project quota trees");
